
#include <stdbool.h>
#include <string.h>
#include "app_button.h"
#include "boards.h"
#include "nrf_gpio.h"
#include "app_timer.h"


#define APP_TIMER_PRESCALER             0  // Value of the RTC1 PRESCALER register.
#define APP_TIMER_MAX_TIMERS            2  // Maximum number of simultaneously created timers. // NORDIC One for debounce, one for long press detect
#define APP_TIMER_OP_QUEUE_SIZE         2  // Size of timer operation queues. 
#define BUTTON_DEBOUNCE_DELAY		        50 // Delay from a GPIOTE event until a button is reported as pushed. 

#define LONG_BUTTON_PRESS_TIME_MS       5000
#define BUTTON_PRESS_TIME_MS            3000

#define MS_TO_TICKS(ms)                 (ms*32768/1000)
static app_timer_id_t                   m_timer_id;

/*
 * Handler to be called when button is pushed.
 * param[in]   pin_no 			The pin number where the event is genereated
 * param[in]   button_action 	Is the button pushed or released
 */
static void button_handler(uint8_t pin_no, uint8_t button_action)
{
    if(button_action == APP_BUTTON_PUSH)
    {
        switch(pin_no)
        {			
            case BUTTON_1:				
               
                app_timer_start(m_timer_id, MS_TO_TICKS(BUTTON_PRESS_TIME_MS), NULL); // NORDIC: Start single shot timer
						    break;
                
            case BUTTON_2:
							  app_timer_start(m_timer_id, MS_TO_TICKS(LONG_BUTTON_PRESS_TIME_MS), NULL); 
						    break;
						
           default:
                break;
        }                                                          
    }
}

/**  
 * Initialize the clock.
 */
void init_clock()
{
    NRF_CLOCK->LFCLKSRC            = (CLOCK_LFCLKSRC_SRC_Xtal << CLOCK_LFCLKSRC_SRC_Pos);
    NRF_CLOCK->EVENTS_LFCLKSTARTED = 0;
    NRF_CLOCK->TASKS_LFCLKSTART    = 1;
    while (NRF_CLOCK->EVENTS_LFCLKSTARTED == 0); // Wait for clock to start
}	

/**
 * Initialize all four LEDs on the nRF51 DK.
 */
void init_leds()
{
    nrf_gpio_range_cfg_input(BUTTON_1, BUTTON_2, NRF_GPIO_PIN_PULLUP);//
    nrf_gpio_range_cfg_output(LED_3, LED_4); 
	  nrf_gpio_pin_set(LED_3); //clear the LED                              
    nrf_gpio_pin_set(LED_4);//clear the LED
                                                                                                                                  
}	

/*
 * Handler to be called on single shot timer event.
 */
static void timer_handler(void * p_context)
{
    if(!nrf_gpio_pin_read(BUTTON_1))
        nrf_gpio_pin_clear(LED_3);  // Turn  LED 1 if button 1 is still pressed
    else
        if(!nrf_gpio_pin_read(BUTTON_2))
			  nrf_gpio_pin_set(LED_3); // Turn off LED 1 if button 2 is still pressed
				 
}
/**@brief Function for initializing the button handler module.
 */
static void buttons_init(void)
{
     uint32_t err_code;
    // Button configuration structure.
    static app_button_cfg_t p_button[] = {  {BUTTON_1, APP_BUTTON_ACTIVE_LOW, NRF_GPIO_PIN_PULLUP, button_handler},
                                          {BUTTON_2, APP_BUTTON_ACTIVE_LOW, NRF_GPIO_PIN_PULLUP, button_handler}};
        
    // Macro for initializing the application timer module.
    // It will handle dimensioning and allocation of the memory buffer required by the timer, making sure that the buffer is correctly aligned. It will also connect the timer module to the scheduler (if specified).
    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_MAX_TIMERS, APP_TIMER_OP_QUEUE_SIZE, NULL);    
    app_timer_create(&m_timer_id, APP_TIMER_MODE_SINGLE_SHOT, timer_handler); // NORDIC Create single shot timer triggering interrupt handler timer_handler once
    
    // Initializing the buttons.
    err_code = app_button_init(p_button, sizeof(p_button) / sizeof(p_button[0]), BUTTON_DEBOUNCE_DELAY);
    APP_ERROR_CHECK(err_code);
                                            
    // Enabling the buttons.										
    err_code = app_button_enable();
    APP_ERROR_CHECK(err_code);
		while(true)
    {
        // Do nothing.
    }
}

/**
 * Function for application main entry.
 */
int main(void)
{	
    init_leds();
    init_clock();
    buttons_init();
    
}

